/*
 * Form.js
 */

"use strict";
class Httpbin {

    static  URL() {
        return 'https://httpbin.org';
    }
    static  Path() {
        return '/post';
    }

    static Run1(name = "FranckBarbier", job = "Trainer") {
        fetch(Httpbin.URL() + Httpbin.Path(), {
            body: JSON.stringify({
                name: name,
                job: job
            }),
            headers: {
                'accept': 'application/json', // -> response
                'content-type': 'application/json; charset=UTF-8'
            },
            method: 'POST'
        }).then(response => {
            response.json().then(result => {
                // 'Httpbin' just returns data for checking:
                window.console.assert(result.json.name === name);
                window.alert("'Httpbin' ('fetch') RUN1: " + JSON.stringify(result.json));
            });
        });
    }

    static Run2(name = "FranckBarbier", job = "Trainer") {
        const form_data = new FormData();
        form_data.append("name", name);
        form_data.append("job", job);
        const request = new XMLHttpRequest();
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                window.console.assert(request.status === 200);
                if (request.responseType === "") // An empty 'responseType' string is treated the same as "text", the default type...
                    window.alert("'ReqRes' ('XMLHttpRequest') RUN2: " + request.responseText);
                if (request.responseType === "json") {
                    const result = JSON.parse(request);
                    window.alert(Object.getOwnPropertyNames(result).join(" - "));
                }
            }
        };
        request.open('POST', Httpbin.URL() + Httpbin.Path(), true);
        request.setRequestHeader('accept', 'application/json');
        request.setRequestHeader('content-type', 'application/x-www-form-urlencoded');
        request.send(form_data);
    }
}


class ReqRes {

    static  URL() {
        return 'https://reqres.in';
    }
    static  Path() {
        return '/api/users';
    }

    static Run1(name = "FranckBarbier", job = "Trainer") {
        fetch(ReqRes.URL() + ReqRes.Path(), {
            body: JSON.stringify({
                name: name,
                job: job
            }),
            headers: {// Mandatory to get 'result' in appropriate shape...
                'accept': 'application/json', // -> response
                'content-type': 'application/json; charset=UTF-8'
            },
            method: 'POST'
        }).then(response => {
            response.json().then(result => {
                window.console.assert(result.name === name); // 'Req-Res' just returns data for "simulation"...
                window.alert("'ReqRes' ('fetch') RUN1: " + JSON.stringify(result));
            });
        });
    }

    static Run2(name = "FranckBarbier", job = "Trainer") {
        const form_data = new FormData(); // Empty...
        form_data.append("name", name);
        form_data.append("job", job);
        const request = new XMLHttpRequest();
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                window.console.assert(request.status === 201); // '201' -> Created
                if (request.responseType === "") // An empty 'responseType' string is treated the same as "text", the default type...
                    window.alert("'ReqRes' ('XMLHttpRequest') RUN2: " + request.responseText);
                if (request.responseType === "json")
                    window.alert(Object.getOwnPropertyNames(JSON.parse(request)).join(" - "));
            }
        };
        request.open('POST', ReqRes.URL() + ReqRes.Path());
        request.setRequestHeader('accept', 'application/json; charset=UTF-8');
        request.setRequestHeader('content-type', 'application/x-www-form-urlencoded'); // Mandatory to avoid CORS...
        request.send(form_data);
    }

    static Run3(name = "FranckBarbier", job = "Trainer") {
        const my_form = window.document.getElementById('my_form'); // Not empty...
        const form_data = new FormData(my_form); // Default content type: 'multipart/form-data'
        window.console.assert(form_data.has("name") && form_data.has("job"));
        form_data.set("name", name);
        form_data.set("job", job);
        fetch(ReqRes.URL() + ReqRes.Path(), {
            body: form_data, /*new URLSearchParams("name=" + name + "&job=" + job),*/
            headers: {
                'accept': 'application/json; charset=UTF-8', // -> response
                'content-type': /*'application/x-www-form-urlencoded; charset=UTF-8'*/'multipart/form-data; charset=UTF-8' // Mandatory to avoid CORS...
            },
            method: 'POST'
        }).then(response => {
            window.console.assert(response.ok && response.status === 201); // '201' -> Created
            response.json().then(result => { // window.console.assert(result.name === name); // It fails...
                window.alert("'ReqRes' ('fetch') RUN3: " + JSON.stringify(result));
            });
        });
    }

    static Run4(name = "FranckBarbier", job = "Trainer") {
        const my_form = window.document.getElementById('my_form'); // Not empty...
        const form_data = new FormData(my_form); // Default content type: 'multipart/form-data'
        window.console.assert(form_data.has("name") && form_data.has("job"));
        form_data.set("name", name);
        form_data.set("job", job);
// https://api.jquery.com/jquery.ajax/
        $.ajax({
            accepts: 'application/json; charset=utf-8',
            url: ReqRes.URL() + ReqRes.Path(),
            type: 'POST',
            data: form_data,
            processData: false, // Tells jQuery not to process data...
            contentType: false   // Tells jQuery not to set 'content-type'
        }).done(result => {
            window.alert("'ReqRes' ('$.ajax') RUN4: " + JSON.stringify(result));
        });
    }
}

window.document.onreadystatechange = function () {
    if (window.document.readyState === "interactive") {
        // window.console.log("DOM just loaded...");
        window.document.forms['my_form'].onsubmit = async(event) => { // Subscription...
            event.preventDefault();
            // window.alert("You just press 'Send form...'");
            const body = new URLSearchParams([...new FormData(event.target).entries()]);
// <form id="my_form" name="my_form" method="post" action="https://reqres.in/api/users">
            const response = await new Response(body).text();
            window.alert(response);
        };
    }
};

function main() {
    //Httpbin.Run1();
    //Httpbin.Run2();
    //ReqRes.Run1();
    //ReqRes.Run2();
    //ReqRes.Run3();
    //ReqRes.Run4();
}
